import React, { useEffect, useState } from "react";
import { useDispatch, useSelector } from "react-redux";
import { connect } from "./redux/blockchain/blockchainActions";
import { fetchData } from "./redux/data/dataActions";
import styled from "styled-components";

const truncate = (input, len) =>
  input.length > len ? `${input.substring(0, len)}...` : input;

function App() {
  const dispatch = useDispatch();
  const blockchain = useSelector((state) => state.blockchain);
  const data = useSelector((state) => state.data);
  const [claimingNft, setClaimingNft] = useState(false);
  const [feedback, setFeedback] = useState(`Click buy to mint your NFT.`);
  const [mintAmount, setMintAmount] = useState(1);
  const [CONFIG, SET_CONFIG] = useState({
    CONTRACT_ADDRESS: "",
    SCAN_LINK: "",
    NETWORK: {
      NAME: "",
      SYMBOL: "",
      ID: 0,
    },
    NFT_NAME: "",
    SYMBOL: "",
    MAX_SUPPLY: 1,
    WEI_COST: 0,
    DISPLAY_COST: 0,
    GAS_LIMIT: 0,
    MARKETPLACE: "",
    MARKETPLACE_LINK: "",
    SHOW_BACKGROUND: false,
  });

  const claimNFTs = () => {
    let cost = CONFIG.WEI_COST;
    let gasLimit = CONFIG.GAS_LIMIT;
    let totalCostWei = String(cost * mintAmount);
    let totalGasLimit = String(gasLimit * mintAmount);
    console.log("Cost: ", totalCostWei);
    console.log("Gas limit: ", totalGasLimit);
    setFeedback(`Minting your ${CONFIG.NFT_NAME}...`);
    setClaimingNft(true);
    blockchain.smartContract.methods
      .mint(mintAmount)
      .send({
        gasLimit: String(totalGasLimit),
        to: CONFIG.CONTRACT_ADDRESS,
        from: blockchain.account,
        value: totalCostWei,
      })
      .once("error", (err) => {
        console.log(err);
        setFeedback("Sorry, something went wrong please try again later.");
        setClaimingNft(false);
      })
      .then((receipt) => {
        console.log(receipt);
        setFeedback(
          `WOW, the ${CONFIG.NFT_NAME} is yours! go visit Elacity to view it.`
        );
        setClaimingNft(false);
        dispatch(fetchData(blockchain.account));
      });
  };

  const decrementMintAmount = () => {
    let newMintAmount = mintAmount - 1;
    if (newMintAmount < 1) {
      newMintAmount = 1;
    }
    setMintAmount(newMintAmount);
  };

  const incrementMintAmount = () => {
    let newMintAmount = mintAmount + 1;
    if (newMintAmount > 10) {
      newMintAmount = 10;
    }
    setMintAmount(newMintAmount);
  };

  const getData = () => {
    if (blockchain.account !== "" && blockchain.smartContract !== null) {
      dispatch(fetchData(blockchain.account));
    }
  };

  const getConfig = async () => {
    const configResponse = await fetch("/config/config.json", {
      headers: {
        "Content-Type": "application/json",
        Accept: "application/json",
      },
    });
    const config = await configResponse.json();
    SET_CONFIG(config);
  };

  useEffect(() => {
    getConfig();
  }, []);

  useEffect(() => {
    getData();
  }, [blockchain.account]);

  return (


<div className="min-h-screen flex flex-col items-center bg-gradient-to-r from-blue-900 to-gray-900 text-white p-6">
  <img alt="logo" src="/config/images/bpunk.svg" className="w-40 md:w-60 mb-8" />
  <div className="flex flex-col md:flex-row md:space-x-8 w-full max-w-6xl items-center gap-6">
    <div className="flex flex-1 justify-center">
      <img alt="example" src="/config/images/example.gif" className="rounded-full w-48 md:w-64 shadow-2xl" />
    </div>
    <div className="flex flex-2 flex-col justify-center items-center bg-gray-800 p-8 rounded-3xl border-4 border-dashed border-secondary shadow-2xl">
      <div className="text-center text-5xl font-extrabold text-white mb-6">
        {data.totalSupply} / {CONFIG.MAX_SUPPLY}
      </div>
      <div className="text-center text-xl mb-4">
        <a className="text-blue-400 hover:text-blue-600" target="_blank" rel="noopener noreferrer" href={CONFIG.SCAN_LINK}>
          {truncate(CONFIG.CONTRACT_ADDRESS, 15)}
        </a>
      </div>
      {Number(data.totalSupply) >= CONFIG.MAX_SUPPLY ? (
        <>
          <div className="text-center text-2xl mb-4">The sale has ended.</div>
          <div className="text-center text-lg mb-4">You can still find {CONFIG.NFT_NAME} on</div>
          <a className="text-blue-400 hover:text-blue-600" target="_blank" rel="noopener noreferrer" href={CONFIG.MARKETPLACE_LINK}>
            {CONFIG.MARKETPLACE}
          </a>
        </>
      ) : (
        <>
          <div className="text-center text-2xl mb-6">
            1 {CONFIG.SYMBOL} costs {CONFIG.DISPLAY_COST} {CONFIG.NETWORK.SYMBOL}.
          </div>
          <div className="text-center text-lg mb-6">Excluding gas fees.</div>
          {blockchain.account === "" || blockchain.smartContract === null ? (
            <div className="flex flex-col items-center">
              <div className="text-center text-lg mb-4">Connect to the {CONFIG.NETWORK.NAME} network</div>
              <button
                className="px-8 py-3 bg-blue-500 text-lg font-semibold rounded-full shadow-lg hover:bg-blue-700 transition duration-200"
                onClick={(e) => {
                  e.preventDefault();
                  dispatch(connect());
                  getData();
                }}
              >
                CONNECT
              </button>
              {blockchain.errorMsg !== "" ? (
                <div className="mt-4 text-red-500 text-center">{blockchain.errorMsg}</div>
              ) : null}
            </div>
          ) : (
            <>
              <div className="text-center text-lg mb-4">{feedback}</div>
              <div className="flex items-center justify-center space-x-6 mb-4">
                <button
                  className="px-6 py-3 bg-red-500 text-lg font-semibold rounded-full shadow-lg hover:bg-red-700 transition duration-200"
                  disabled={claimingNft}
                  onClick={(e) => {
                    e.preventDefault();
                    decrementMintAmount();
                  }}
                >
                  -
                </button>
                <div className="text-2xl">{mintAmount}</div>
                <button
                  className="px-6 py-3 bg-green-500 text-lg font-semibold rounded-full shadow-lg hover:bg-green-700 transition duration-200"
                  disabled={claimingNft}
                  onClick={(e) => {
                    e.preventDefault();
                    incrementMintAmount();
                  }}
                >
                  +
                </button>
              </div>
              <button
                className="px-8 py-3 bg-yellow-500 text-lg font-semibold rounded-full shadow-lg hover:bg-yellow-700 transition duration-200"
                disabled={claimingNft}
                onClick={(e) => {
                  e.preventDefault();
                  claimNFTs();
                  getData();
                }}
              >
                {claimingNft ? "BUSY" : "BUY"}
              </button>
            </>
          )}
        </>
      )}
    </div>
    <div className="flex flex-1 justify-center mb-8 md:mb-0">
      <img alt="example" src="/config/images/example.gif" className="rounded-full w-48 md:w-64 shadow-2xl transform scale-x-[-1]" />
    </div>
  </div>
  <div className="mt-12 text-center max-w-2xl">
    <p className="mb-6 text-lg">
      Please make sure you are connected to the right network ({CONFIG.NETWORK.NAME} Mainnet) and the correct address. Please note: Once you make the purchase, you cannot undo this action.
    </p>
    <p className="text-lg">
      We have set the gas limit to {CONFIG.GAS_LIMIT} for the contract to successfully mint your NFT. We recommend that you don't lower the gas limit.
    </p>
  </div>
</div>





   
  );
}

export default App;
